

//		Copyright © 1999		Matthew S. Chartier	(mattc@ic.net)
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
// 		Portions of this file copyright xxxx



#include <KernelExport.h>
#include <ISA.h>

#include "ppa_lowlevel.h"

extern isa_module_info* GLB_Bus;


static int      ppa_base       = IOPORT;	/* parallel port address    */
static int      ppa_speed_high = 0;    		/* port delay in data phase */
static int      ppa_speed_low  = 1;   		/* port delay otherwise     */
static int      ppa_nybble     = 0;      	/* don't force nybble mode  */

static int    	ppa_busy = 0;
static int      ppa_abort_flag = 0;
static int      ppa_error_code = DID_OK;


int				ppa_port_delay = 1;

extern isa_module_info* GLB_Bus;


static sem_id lock = B_ERROR;
static bool busy_ = false;

status_t lockIO(void)
	{
	while(busy_)
		snooze(1);
		
	busy_ = true;		
	return(B_NO_ERROR);
	}
	
status_t unlockIO(void)
	{
	busy_ = false;
	}	

int barrier()
	{
	static int g = 1;
	g << 1;
	return(g);
	}

void udelay(ulong delay)
	{
	status_t previousPriority = set_thread_priority(find_thread(NULL),B_REAL_TIME_PRIORITY);
	snooze(delay);
	set_thread_priority(find_thread(NULL),previousPriority);
	}


void out_p(short port,char byte)
	{
	GLB_Bus->write_io_8(ppa_base+port,byte);
	if(ppa_port_delay == ppa_speed_low)
		udelay(1);
	}

char in_p(short port)
	{
	char c;

	c = GLB_Bus->read_io_8(ppa_base+port);
	return(c);
	}

void ppa_d_pulse(char b)
	{
	out_p(0,b);
    out_p(2,0xc); out_p(2,0xe); out_p(2,0xc); out_p(2,0x4); out_p(2,0xc);
	}

void ppa_disconnect(void)
	{
	ppa_d_pulse(0);
    ppa_d_pulse(0x3c);
    ppa_d_pulse(0x20);
    ppa_d_pulse(0xf);
	}

void ppa_c_pulse(char b)
	{
	out_p(0,b);
    out_p(2,0x4); out_p(2,0x6); out_p(2,0x4); out_p(2,0xc);
	}

void ppa_connect(void)
	{
	ppa_c_pulse(0);
    ppa_c_pulse(0x3c);
    ppa_c_pulse(0x20);
    ppa_c_pulse(0x8f);
	}

void ppa_do_reset(void)
	{
	out_p(2,0);
	snooze(ppa_port_delay);
	}

int ppa_init(void)
	{
	char    r, s;

    out_p(0,0xaa); 
    if (in_p(0) != (char) 0xaa) return 1; 
    ppa_disconnect();
    ppa_connect();
    out_p(2,0x6); 
    if ((in_p(1) & 0xf0) != 0xf0) return 2; 
    out_p(2,0x4);
    if ((in_p(1) & 0xf0) != 0x80) return 3; 
    ppa_disconnect();
    s = in_p(2);
    out_p(2,0xec);
    out_p(0,0x55); 
    r = in_p(0);    
    if (r != (char) 0xff)
    	{ 
		ppa_nybble = 1;
        if (r != (char) 0x55) return 4; 
        out_p(0,0xaa); if (in_p(0) != (char) 0xaa) return 5; 
        }
 	
 	out_p(2,s);
    ppa_connect();
    out_p(0,0x40); out_p(2,0x8); out_p(2,0xc);
    ppa_disconnect();

    return 0;
	}

char ppa_select( int  initiator,int target)
	{
	char    r;
    int     k;

    r = in_p(1);
    out_p(0,(1<<target)); out_p(2,0xe); out_p(2,0xc);
    out_p(0,(1<<initiator)); out_p(2,0x8);

    k = 0;
    while ( !(r = (in_p(1) & 0xf0)) && (k++ < PPA_SELECT_TMO)) 
    	barrier();
    return r;
	}

char ppa_wait( void ) 
	{
	int     k;
    char    r;

    ppa_error_code = DID_OK;
    k = 0;
    while (!((r = in_p(1)) & 0x80) && (k++ < PPA_SPIN_TMO) && !ppa_abort_flag  ) barrier();
                        
    if (ppa_abort_flag)
    	{
        if (ppa_abort_flag == 1) 
        	ppa_error_code = DID_ABORT;
        else
         	{
         	ppa_do_reset();
            ppa_error_code = DID_RESET;
            }
     	ppa_disconnect();
        return 0;
        }
 	if (k >= PPA_SPIN_TMO)
 		{ 
        ppa_error_code = DID_TIME_OUT;
        ppa_disconnect();
        return 0;               /* command timed out */
        }
  	return (r & 0xf0);
	}

int ppa_start(Scsi_Cmnd* cmd )
	{
	int     k;
    ppa_error_code = DID_OK;
    ppa_abort_flag = 0;

    if (cmd->target == PPA_INITIATOR)
    	{
         ppa_error_code = DID_BAD_TARGET;
         return 0;
        }
   	ppa_connect();
    if (!ppa_select(PPA_INITIATOR,cmd->target))
    	{
        ppa_disconnect();
        ppa_error_code = DID_NO_CONNECT;
        return 0;
        }
	out_p(2,0xc);

    for (k=0; k < cmd->cmd_len; k++)
    	{        /* send the command */
        if (!ppa_wait()) return 0;
        out_p(0,cmd->cmnd[k]);
        out_p(2,0xe);
        out_p(2,0xc);
        }

        return 1;
	}

int ppa_completion(Scsi_Cmnd* cmd)
	{
	char    r, l, h, v;
    int     dir, cnt, blen, fast, bulk;
    char    *buffer;

    if (!(r = ppa_wait())) return 0;

    v = cmd->cmnd[0];
    bulk = ((v==READ_6)||(v==READ_10)||(v==WRITE_6)||(v==WRITE_10));
    buffer = cmd->request_buffer;
    blen = cmd->request_bufflen;
    cnt = 0;  dir = 0;
    if (r == (char) 0xc0) dir = 1;  /* d0 = read c0 = write f0 = status */

    ppa_port_delay = ppa_speed_high;

    while (r != (char) 0xf0)
    	{
        if (((r & 0xc0) != 0xc0 ) || (cnt >= blen))
        	{
            ppa_disconnect();
            ppa_error_code = DID_ERROR;
            return 0;
            }
       	fast = bulk && ((blen - cnt) >= PPA_SECTOR_SIZE);
        if (dir)
        	do
        		{
            	out_p(0,buffer[cnt++]);
            	out_p(2,0xe); out_p(2,0xc);
            	if (!fast)
            		break;
          		}
     		while (cnt % PPA_SECTOR_SIZE);
      	else
      		{  
            if (ppa_nybble)
            	do
            		{
                    out_p(2,0x4); h = in_p(1); 
                    out_p(2,0x6); l = in_p(1);
                    v = ((l >> 4) & 0x0f) + (h & 0xf0);

                    buffer[cnt++] = v;
                    if (!fast)
                    	break;
                  	}
            	while (cnt % PPA_SECTOR_SIZE);
          	else
          		do
          			{
                    out_p(2,0x25); v = in_p(0); out_p(2,0x27);
                    buffer[cnt++] = v;
                    if (!fast)
                    	break;
                  	}
               	while (cnt % PPA_SECTOR_SIZE);
     	
     	if (!ppa_nybble)
     		{
            out_p(2,0x5); out_p(2,0x4);
            }
	 	out_p(2,0xc);
        }
  	if (!(r = ppa_wait()))
  		return 0;
	}

    ppa_port_delay = ppa_speed_low;

    out_p(2,0x4);            /* now read status byte */
    h = in_p(1);
    out_p(2,0x6);
    l = in_p(1);
    out_p(2,0xc);
    r = ((l >> 4) & 0x0f) + (h & 0xf0);

    out_p(2,0xe); out_p(2,0xc);
    ppa_disconnect();

    return (r & STATUS_MASK);
	}


int scsiSendCommand(Scsi_Cmnd* cmd)
	{
	int ret = -1;

	if(ppa_start(cmd))
		{
		if(ppa_wait())
			ret = ppa_completion(cmd) + (ppa_error_code << 16);		
		}

	return(ret);
	}


status_t ppa_SendCommand(void* cmd,uint cmdlen,void* data,uint datalen)
	{
	status_t ret = B_ERROR;
   	Scsi_Cmnd internalCmd;

	if(lockIO() == B_NO_ERROR)
		{
		memset(&internalCmd,0,sizeof(internalCmd));
		internalCmd.target = PPA_ZIP_LUN;
   		memcpy(&internalCmd.cmnd[0],cmd,cmdlen);
   		internalCmd.cmd_len = cmdlen;

   		memcpy((char*)internalCmd.request_buffer,(char*)data,datalen);
   		internalCmd.request_bufflen = datalen;
	
   		ret = scsiSendCommand(&internalCmd);
    	memcpy(data,internalCmd.request_buffer,datalen);
		unlockIO();
		}

   return(ret);
   }

